%module(directors="1") rave

%{

#include <rave/VertexFactory.h>
#include <rave/Track.h>
#include <rave/Vertex.h>
#include <rave/VacuumPropagator.h>
#include <rave/ConstantMagneticField.h>
#include <rave/Histogram.h>
#include <rave/CalibrationHistogram.h>
#include <rave/Calibration.h>
#include <rave/Version.h>

#ifdef WITH_FLAVORTAGGING
#include <rave/FlavorTagFactory.h>
#endif

#ifdef WITH_KINEMATICS
#include <rave/Vector4D.h>
#include <rave/Vector7D.h>
#include <rave/Covariance4D.h>
#include <rave/Covariance7D.h>
#include <rave/KinematicParticle.h>
#include <rave/TransientTrackKinematicParticle.h>
#include <rave/VirtualKinematicParticle.h>
#include <rave/KinematicVertex.h>
#include <rave/KinematicConstraint.h>
#include <rave/MultipleKinematicConstraint.h>
#include <rave/KinematicConstraintBuilder.h>
#include <rave/KinematicTree.h>
#include <rave/KinematicTreeFactory.h>
#endif

#include "RaveBase/Converters/interface/RaveStreamers.h"

%}

%feature("director");

%rename(RaveTrack) rave::Track;
%rename(RaveVertexFactory) rave::VertexFactory;
%rename(RaveVertex) rave::Vertex;
%rename(RaveVector6D) rave::Vector6D;
%rename(RaveVector3D) rave::Vector3D;
%rename(RavePoint3D) rave::Point3D;
%rename(RaveCovariance6D) rave::Covariance6D;
%rename(RaveCovariance3D) rave::Covariance3D;
%rename(RaveCharge) rave::Charge;
%rename(RaveMagneticField) rave::MagneticField;
%rename(RavePropagator) rave::Propagator;
%rename(RaveEllipsoid3D) rave::Ellipsoid3D;
%rename(RaveVacuumPropagator) rave::VacuumPropagator;
%rename(RaveConstantMagneticField) rave::ConstantMagneticField;
%rename(RaveVersion) rave::Version;
%rename(RaveFlavorTagFactory) rave::FlavorTagFactory;

#ifdef WITH_KINEMATICS
%rename(RaveVector4D) rave::Vector4D;
%rename(RaveVector7D) rave::Vector7D;
%rename(RaveCovariance4D) rave::Covariance4D;
%rename(RaveCovariance7D) rave::Covariance7D;
%rename(RaveKConstraint) rave::KinematicConstraint;
%rename(RaveKMultipleConstraint) rave::MultipleKinematicConstraint;
%rename(RaveKConstraintBuilder) rave::KinematicConstraintBuilder;
%rename(RaveKParticle) rave::KinematicParticle;
%rename(RaveKTransientTrackParticle) rave::TransientTrackKinematicParticle;
%rename(RaveKVirtualParticle) rave::VirtualKinematicParticle;
%rename(RaveKTree) rave::KinematicTree;
%rename(RaveKTreeFactory) rave::KinematicTreeFactory;
%rename(RaveKVertex) rave::KinematicVertex;
#endif

%include "std_string.i"
%include "std_pair.i"
%include "std_vector.i"

%include <rave/Charge.h>
%include <rave/Point3D.h>
%include <rave/Vector3D.h>
%include <rave/Vector6D.h>
%include <rave/Covariance3D.h>
%include <rave/Covariance6D.h>
%include <rave/MagneticField.h>
%include <rave/ConstantMagneticField.h>
%include <rave/Propagator.h>
%include <rave/VacuumPropagator.h>
%include <rave/Ellipsoid3D.h>
%include <rave/Histogram.h>
%include <rave/CalibrationHistogram.h>
%include <rave/Calibration.h>
%include <rave/Version.h>

#ifdef WITH_KINEMATICS
%include <rave/Vector4D.h>
%include <rave/Vector7D.h>
%include <rave/Covariance4D.h>
%include <rave/Covariance7D.h>
%include <rave/KinematicConstraint.h>
%include <rave/MultipleKinematicConstraint.h>
%include <rave/KinematicConstraintBuilder.h>
%include <rave/KinematicParticle.h>
%include <rave/KinematicVertex.h>
%include <rave/TransientTrackKinematicParticle.h>
%include <rave/VirtualKinematicParticle.h>
%include <rave/KinematicTree.h>
%include <rave/KinematicTreeFactory.h>
#endif

#ifdef WITH_FLAVORTAGGING
%include <rave/FlavorTagFactory.h>
#endif

//%feature("notabstract") RaveMagneticField;
//%feature("notabstract") RaveConstantMagneticField;
//%feature("notabstract") rave::MagneticField;
//%feature("notabstract") rave::ConstantMagneticField;

namespace std {
   %template(RaveTrackContainer) vector<rave::Track>;
   %template(RaveVertexContainer) vector<rave::Vertex>;
#ifdef WITH_KINEMATICS
   %template(RaveKParticleContainer) vector<rave::KinematicParticle>;
   %template(RaveKTreeContainer) vector<rave::KinematicTree>;
#endif
};

namespace rave {
    class Track
    {
        public:
            Track( const Vector6D &, const Covariance6D &, Charge,
                   double chi2, double ndof,
                   int originaltrack, std::string tag="" );
            Track();
            signed charge() const;
            const Vector6D & state() const;
            const Covariance6D & error() const;
            void * originalObject() const;
            std::string tag() const;
            int id() const;
            bool isValid() const;
            std::vector < Track > components() const;
    };

    class Vertex
    {
        public:
            Vertex();
            Vertex( const Point3D &, const Covariance3D &,
                    const std::vector < std::pair < double, Track > > & tracks, double ndf, double chi2 );
            Vertex( const Point3D &, const Covariance3D &,
                    const std::vector < Track > & tracks, double ndf, double chi2 );
            const Point3D & position() const;
            const Covariance3D & error() const;
            const std::vector < std::pair < double, Track > > & weightedTracks() const;
            std::vector < Track > tracks() const;
            double ndf() const;
            double chiSquared() const;
            int id() const;
            bool isValid() const;
            std::vector < Vertex > components() const;
    };

    class VertexFactory
    {
        public:
            VertexFactory ( const rave::MagneticField & m = ConstantMagneticField(),
                      const rave::Propagator & p = VacuumPropagator(),
                      const std::string & method = "default",
                      int verbosity=1 );
            
            VertexFactory ( const rave::MagneticField &,
                      const rave::Propagator &,
                      const rave::Ellipsoid3D & beamspot,
                      const std::string & method = "default",
                      int verbosity=1 );

            VertexFactory ( const VertexFactory & o );
            ~VertexFactory();
            VertexFactory operator= ( const VertexFactory & o );

            std::vector < Vertex > create ( const std::vector < Track > &, bool use_beamspot=false ) const;
            std::vector < Vertex > create ( const std::vector < Track > &, 
                                            const std::string & method, bool use_beamspot=false ) const;

            const rave::Propagator & getPropagator() const;
            const rave::MagneticField & getMagneticField() const;
            const rave::Ellipsoid3D & getBeamspot() const;
            bool hasBeamspot() const;
            const int verbosity() const;
            const std::string & method() const;
            void setDefaultMethod ( const std::string & );
            const RaveReconstructor & reconstructor() const;

    };
};

STRINGHELPER(rave::Point3D);
STRINGHELPER(rave::Vertex);
STRINGHELPER(rave::Track);
STRINGHELPER(rave::Vector6D);

#ifdef WITH_KINEMATICS
STRINGHELPER(rave::Vector4D);
STRINGHELPER(rave::Vector7D);
#endif // WITH_KINEMATICS

#ifdef WITH_FLAVORTAGGING
STRINGHELPER(combsv::CombinedSVCalibrationCategory);
#endif // WITH_FLAVORTAGGING

