#include "DataFormats/VertexReco/interface/Vertex.h" 

reco::Vertex::Vertex () {}

reco::Vertex::Vertex ( const Point & p, const CovarianceMatrix & c, double chi2, double ndf,
                       size_t size ) :
  p_(p), c_(c), chi2_(chi2), ndf_(ndf), size_(size)
{}

const reco::Vertex::Point & reco::Vertex::position() const
{
  return p_;
}

double reco::Vertex::x() const
{
  return p_.x();
}

double reco::Vertex::y() const
{
  return p_.y();
}

double reco::Vertex::z() const
{
  return p_.z();
}

reco::Vertex::CovarianceMatrix reco::Vertex::covariance() const
{
  return c_;
}

double reco::Vertex::chi2() const
{
  return chi2_;
}

double reco::Vertex::ndof() const
{
  return ndf_;
}

void reco::Vertex::removeTracks()
{
  // does nothing, we dont keep tracks
}

size_t reco::Vertex::tracksSize() const
{
  return size_;
}

reco::track_iterator reco::Vertex::tracks_begin() const
{
  return tracks_.begin();
}

void reco::Vertex::add ( const reco::TrackRef & r, double w )
{
  tracks_.push_back ( r );
}

reco::track_iterator reco::Vertex::tracks_end() const
{
  return tracks_.end();
}
