#include "RaveBase/Converters/interface/CmsToRaveObjects.h"
#include "RaveBase/RaveInterface/rave/Vector6D.h"
#include "RaveBase/RaveInterface/rave/Covariance6D.h"
#include "TrackingTools/TrajectoryParametrization/interface/CartesianTrajectoryError.h"
#include "RaveBase/Converters/interface/RaveStreamers.h"
#include <limits>
#include <ostream>
#include "boost/cast.hpp"

using namespace std;
namespace
{
rave::Track getNr ( const vector < rave::Track > & trks, int id )
{
  for ( vector< rave::Track >::const_iterator i = trks.begin();
        i != trks.end() ; ++i )
  {
    if ( i->id() == id ) return *i;
  }
  cout << "[CmsToRaveObjects] no track nr " << id << "?!?" << endl;
  exit ( 0 );
}

typedef numeric_limits<float> num;
}

rave::Track CmsToRaveObjects::convert ( const reco::TransientTrack & t, int id ) const
{
  /*
  rave::Vector6D st=convert ( t.impactPointState().globalParameters() );
  rave::Covariance6D err = convert ( t.impactPointState().cartesianParameters() );
  return rave::Track();
  */
  ostringstream ostr;
  ostr << "id" << id;
  return convert ( t.impactPointState(), t.chi2(), t.ndof(), 0, ostr.str() );
}

rave::Track CmsToRaveObjects::convert ( const TrajectoryStateOnSurface & s, double chi2,
                                        double ndf, void * p, const string & tag ) const
{
  rave::Vector6D st = convert ( s.globalParameters() );
  rave::Covariance6D err = convert ( s.cartesianError() );
  return rave::Track ( st, err, s.charge(), chi2, ndf, p, tag );
}

rave::Track CmsToRaveObjects::convert ( const FreeTrajectoryState & s ) const
{
  rave::Vector6D st = convert ( s.parameters() );
  rave::Covariance6D err = convert ( s.cartesianError() );
  return rave::Track ( st, err, s.charge(), num::quiet_NaN() /*chi2*/,
                       num::quiet_NaN () /*ndof*/, 0, "" );
}

rave::Vector6D CmsToRaveObjects::convert ( const GlobalTrajectoryParameters & p ) const
{
  return rave::Vector6D ( p.position().x(), p.position().y(), p.position().z(),
                          p.momentum().x(), p.momentum().y(), p.momentum().z() );

}

rave::Covariance3D CmsToRaveObjects::convert ( const GlobalError & e ) const
{
  return rave::Covariance3D ( e.cxx(), e.cyx(), e.czx(),
                              e.cyy(), e.czy(), e.czz() );
}

rave::Covariance6D CmsToRaveObjects::convert ( const CartesianTrajectoryError & e ) const
{
  return rave::Covariance6D ( e.matrix_old() [0][0], e.matrix_old() [0][1], e.matrix_old() [0][2],
                              e.matrix_old() [1][1], e.matrix_old() [1][2], e.matrix_old() [2][2],
                              e.matrix_old() [0][3], e.matrix_old() [0][4], e.matrix_old() [0][5],
                              e.matrix_old() [1][3], e.matrix_old() [1][4], e.matrix_old() [1][5],
                              e.matrix_old() [2][3], e.matrix_old() [2][4], e.matrix_old() [2][5],
                              e.matrix_old() [3][3], e.matrix_old() [3][4], e.matrix_old() [3][5],
                              e.matrix_old() [4][4], e.matrix_old() [4][5], e.matrix_old() [5][5] );
  /*
  return rave::Covariance6D ( e.matrix()(0,0), e.matrix()(0,1), e.matrix()(0,2),
                              e.matrix()(1,1), e.matrix()(1,2), e.matrix()(2,2),
                              e.matrix()(0,3), e.matrix()(0,4), e.matrix()(0,5),
                              e.matrix()(1,3), e.matrix()(1,4), e.matrix()(1,5),
                              e.matrix()(2,3), e.matrix()(2,4), e.matrix()(2,5),
                              e.matrix()(3,3), e.matrix()(3,4), e.matrix()(3,5),
                              e.matrix()(4,4), e.matrix()(4,5), e.matrix()(5,5) );
                              */
}

rave::Vertex CmsToRaveObjects::convert ( const TransientVertex & tv,
    const vector < rave::Track > & trks ) const
{
  // cout << "[CmsToRaveObjects] begin conversion!" << endl;
  vector < pair < double, rave::Track > > tracks;

  map<reco::TransientTrack, float > wm;

  if ( tv.hasTrackWeight() )
  {
    wm = tv.weightMap();
  }
  else
  {
    vector < reco::TransientTrack > trks = tv.originalTracks();
    for ( vector< reco::TransientTrack >::const_iterator i = trks.begin();
          i != trks.end() ; ++i )
    {
      wm[*i] = 1.0;
    }
  }

  for ( map<reco::TransientTrack, float >::iterator iter = wm.begin();
        iter != wm.end();
        iter++ )
  {
    // cout << "[CmsToRaveObjects] track: #" << iter->first.id() << endl;
    tracks.push_back ( pair< double, rave::Track > ( iter->second, getNr ( trks, iter->first.id() ) ) );
  }

  double ndf = tv.degreesOfFreedom();
  double chi2 = tv.totalChiSquared();
  /*
  cout << "[CmsToRaveObjects] chi2=" << chi2 << "-"
       << static_cast<double>( tv.totalChiSquared() ) << endl;
       */

  if ( ! ( tv.hasRefittedTracks() ) )
  {
    return rave::Vertex ( rave::Point3D ( tv.position().x(),
                                          tv.position().y(),
                                          tv.position().z() ),
                          CmsToRaveObjects().convert ( tv.positionError() ),
                          tracks, ndf, chi2 );
  }

  vector < pair < double, rave::Track > > ftracks;
  // cout << "[CmsToRaveObjects] FIXME!!" << endl;
  vector < reco::TransientTrack > rfs = tv.refittedTracks();
  vector < reco::TransientTrack > otrks = tv.originalTracks();

  vector < reco::TransientTrack >::const_iterator oit = otrks.begin();

  for ( vector< reco::TransientTrack >::const_iterator i = rfs.begin(); i != rfs.end() ; ++i )
  {
    /*
    cout << "[CmsToRaveObjects] refitted track: p=" << i->impactPointState().globalMomentum() << endl;
    cout << "[CmsToRaveObjects] original track: p=" << oit->impactPointState().globalMomentum() << endl;
    */
    // cout << "[CmsToRaveObjects] wm=" << wm[*i] << endl;
    ftracks.push_back ( pair< double, rave::Track > ( wm[*oit], convert ( *i ) ) );
    oit++;
  }

  return rave::Vertex ( rave::Point3D ( tv.position().x(),
                                        tv.position().y(),
                                        tv.position().z() ),
                        CmsToRaveObjects().convert ( tv.positionError() ),
                        tracks, ftracks, ndf, chi2 );
  /*
                 static_cast<double>( tv.degreesOfFreedom() ),
                 static_cast<double>( tv.totalChiSquared() ) ); */
}

rave::Vector3D CmsToRaveObjects::convert ( const GlobalPoint & p ) const
{
  return rave::Vector3D ( boost::numeric_cast<double> ( p.x() ),
                          boost::numeric_cast<double> ( p.y() ),
                          boost::numeric_cast<double> ( p.z() ) );
}

rave::Vector3D CmsToRaveObjects::convert ( const GlobalVector & v ) const
{
  return rave::Vector3D ( boost::numeric_cast<double> ( v.x() ),
                          boost::numeric_cast<double> ( v.y() ),
                          boost::numeric_cast<double> ( v.z() ) );
}

#ifdef WITH_KINEMATICS

rave::Vector7D CmsToRaveObjects::convert ( const ::KinematicParameters & p ) const
{
  return rave::Vector7D ( CmsToRaveObjects().convert ( p.position() ),
                          CmsToRaveObjects().convert ( p.momentum() ),
                          p.vector() ( 7 ) );
}

rave::Covariance7D CmsToRaveObjects::convert ( const ::KinematicParametersError & e ) const
{
  return rave::Covariance7D (
           e.matrix() [0][0], e.matrix() [0][1], e.matrix() [0][2],
           e.matrix() [1][1], e.matrix() [1][2], e.matrix() [2][2],
           e.matrix() [0][3], e.matrix() [0][4], e.matrix() [0][5],
           e.matrix() [1][3], e.matrix() [1][4], e.matrix() [1][5],
           e.matrix() [2][3], e.matrix() [2][4], e.matrix() [2][5],
           e.matrix() [3][3], e.matrix() [3][4], e.matrix() [3][5],
           e.matrix() [4][4], e.matrix() [4][5], e.matrix() [5][5],
           e.matrix() [0][6], e.matrix() [1][6], e.matrix() [2][6],
           e.matrix() [3][6], e.matrix() [4][6], e.matrix() [5][6],
           e.matrix() [6][6] );
}

#endif
