#ifndef _RaveCovariance6D_H_
#define _RaveCovariance6D_H_

#include "boost/operators.hpp"

namespace rave
{

/**
 * @class RaveCovariance6D defines the input to the rave library.
 */

class Covariance6D :
      boost::equality_comparable<Covariance6D>
{
  public:

    /**
     *  Create a RaveCovariance6D from a 6-dimensional euclidean trajectory state
     *  plus covariance matrix, plus charge
     */
    Covariance6D ( double dxx, double dxy, double dxz,
                   double dyy, double dyz, double dzz,
                   double dxpx, double dxpy, double dxpz,
                   double dypx, double dypy, double dypz,
                   double dzpx, double dzpy, double dzpz,
                   double dpxpx, double dpxpy, double dpxpz,
                   double dpypy, double dpypz, double dpzpz );

    /**
     *  empty error matrix
     */
    Covariance6D();

    double dxx() const;
    double dxy() const;
    double dxz() const;
    double dyy() const;
    double dyz() const;
    double dzz() const;

    double dxpx() const;
    double dxpy() const;
    double dxpz() const;
    double dypx() const;
    double dypy() const;
    double dypz() const;
    double dzpx() const;
    double dzpy() const;
    double dzpz() const;

    double dpxpx() const;
    double dpxpy() const;
    double dpxpz() const;
    double dpypy() const;
    double dpypz() const;
    double dpzpz() const;

    virtual bool equal ( const Covariance6D & other ) const;

  private:
    double thedxx;
    double thedxy;
    double thedxz;
    double thedyy;
    double thedyz;
    double thedzz;

    double thedxpx;
    double thedxpy;
    double thedxpz;
    double thedypx;
    double thedypy;
    double thedypz;
    double thedzpx;
    double thedzpy;
    double thedzpz;

    double thedpxpx;
    double thedpxpy;
    double thedpxpz;
    double thedpypy;
    double thedpypz;
    double thedpzpz;
};

} // namespace rave

bool operator== ( const rave::Covariance6D & lhs, const rave::Covariance6D & rhs );

#endif
