//
// C++ Interface: RaveCovariance7D
//
// Description:
//
//
// Author: Fabian Moser <fabiamos@gmail.com>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef RAVECOVARIANCE7D_H
#define RAVECOVARIANCE7D_H

#include <rave/Covariance6D.h>

#include <ostream>

namespace rave
{

/**
 * rave::Covariance7D holds the error data for kinematical fit data objects,
 * namely rave::KinematicParticle and family. It corresponds *to the internal
 * KinematicParametersError.
 *
 * @author Fabian Moser <fabiamos@gmail.com>
 */

class Covariance7D :
      public Covariance6D,
      boost::equality_comparable<Covariance7D>
{

  public:
    Covariance7D ( double dxx, double dxy, double dxz,
                   double dyy, double dyz, double dzz,
                   double dxpx, double dxpy, double dxpz,
                   double dypx, double dypy, double dypz,
                   double dzpx, double dzpy, double dzpz,
                   double dpxpx, double dpxpy, double dpxpz,
                   double dpypy, double dpypz, double dpzpz,
                   double dxm, double dym, double dzm,
                   double dpxm, double dpym, double dpzm,
                   double dmm );

    Covariance7D ( Covariance6D space_error,
                   double dxm, double dym, double dzm,
                   double dpxm, double dpym, double dpzm,
                   double dmm );

    Covariance7D();

    double dxm() const;
    double dym() const;
    double dzm() const;
    double dpxm() const;
    double dpym() const;
    double dpzm() const;
    double dmm() const;

    bool equal ( const Covariance7D & other ) const;

  private:
    double thedxm;
    double thedym;
    double thedzm;
    double thedpxm;
    double thedpym;
    double thedpzm;
    double thedmm;
};

} // namespace rave

bool operator== ( const rave::Covariance7D & lhs, const rave::Covariance7D & rhs );
std::ostream& operator<< (std::ostream& os, const rave::Covariance7D & );

#endif
