#ifndef _RAVEVECTOR6D_H_
#define _RAVEVECTOR6D_H_

#include <rave/Vector3D.h>
#include <rave/Point3D.h>

#include "boost/operators.hpp"

namespace rave
{

/** \class RaveVector6D a 6-dimensional state
 */

class Vector6D :
    boost::equality_comparable<Vector6D>
{
  public:

    Vector6D();

    Vector6D( double x, double y, double z,
              double px, double py, double pz );

    Vector6D( const Point3D & position,
              const Vector3D & momentum );

    double x() const;
    double y() const;
    double z() const;

    double px() const;
    double py() const;
    double pz() const;

    Vector3D momentum() const;
    Point3D position() const;

    virtual bool equal(const Vector6D & other) const;

  private:
    double thex;
    double they;
    double thez;
    double thepx;
    double thepy;
    double thepz;
};

} // namespace rave

bool operator==(const rave::Vector6D & lhs, const rave::Vector6D & rhs);

#endif

