//
// C++ Interface: RaveVector7D
//
// Description:
//
// The Vector7D objects hold kinematic particle data. They extend the
// Vector6D from the geometrical fit to the necessary 7th dimension.
//
// Author: Fabian Moser <fabiamos@gmail.com>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef RAVEVECTOR7D_H
#define RAVEVECTOR7D_H

#include <rave/Vector6D.h>

#include <ostream>

namespace rave
{

/**
 * @class Vector7D adds the mass/energy information to a Vector6D.
 *
 * @author Fabian Moser <fabiamos@gmail.com>
 */

class Vector7D :
    public Vector6D,
    boost::equality_comparable<Vector7D>
{

  public:
    Vector7D();

    Vector7D ( double x, double y, double z,
               double px, double py, double pz,
               double m );

    Vector7D ( const Point3D & position,
               const Vector3D & momentum,
               double m );

    double m() const;

    bool equal(const Vector7D & other) const;

  private:
    double them;

};

} // namespace rave

bool operator==(const rave::Vector7D & lhs, const rave::Vector7D & rhs);
std::ostream& operator<< (std::ostream& os, const rave::Vector7D & );

#endif
