#include "RaveBase/RaveInterface/rave/BasicKinematicParticle.h"

#include "RaveBase/Converters/interface/CmsToRaveObjects.h"
#include "RaveBase/Converters/interface/RaveStreamers.h"
#include "RaveBase/RaveEngine/interface/RaveId.h"

#include "RecoVertex/KinematicFitPrimitives/interface/RefCountedKinematicParticle.h"

#include <iostream>

#include "boost/assert.hpp"
#include "boost/cast.hpp"

namespace rave
{

const Vector7D & BasicKinematicParticle::fullstate() const
{
  BOOST_ASSERT (
    CmsToRaveObjects().convert (
      boost::any_cast< ::RefCountedKinematicParticle > ( theParticle )->currentState().kinematicParameters()
    ) ==
    theState );
  return theState;
}

const Covariance7D & BasicKinematicParticle::fullerror() const
{
  BOOST_ASSERT (
    CmsToRaveObjects().convert (
      boost::any_cast< ::RefCountedKinematicParticle > ( theParticle )->currentState().kinematicParametersError()
    ) ==
    theError );
  return theError;
}

const Vector6D & BasicKinematicParticle::state() const
{
  return fullstate();
}

const Covariance6D & BasicKinematicParticle::error() const
{
  return fullerror();
}

Charge BasicKinematicParticle::charge() const
{
  BOOST_ASSERT (
    boost::numeric_cast<Charge> (
      boost::any_cast< ::RefCountedKinematicParticle > ( theParticle )->currentState().particleCharge()
    ) ==
    theCharge );
  return theCharge;
}

double BasicKinematicParticle::chi2() const
{
  BOOST_ASSERT (
    boost::numeric_cast<double> (
      boost::any_cast< ::RefCountedKinematicParticle > ( theParticle )->chiSquared()
    ) ==
    theChi2 );
  return theChi2;
}

double BasicKinematicParticle::ndof() const
{
  BOOST_ASSERT (
    boost::numeric_cast<double> (
      boost::any_cast< ::RefCountedKinematicParticle > ( theParticle )->degreesOfFreedom()
    ) ==
    theNdof );
  return theNdof;
}

boost::shared_ptr< MagneticField > BasicKinematicParticle::magneticField() const
{
  return theField;
}

/*
KinematicTree BasicKinematicParticle::correspondingTree() const
{
  return theTree;
}
*/

boost::shared_ptr< rave::KinematicConstraint > BasicKinematicParticle::lastConstraint() const
{
  return theConstraint;
}

int BasicKinematicParticle::id() const
{
  return theId;
}

boost::any BasicKinematicParticle::link() const
{
  return theLink;
}

std::string BasicKinematicParticle::tag() const
{
  return theTag;
}

bool BasicKinematicParticle::isValid() const
{
  return theIsValid;
}

BasicKinematicParticle * rave::BasicKinematicParticle::clone() const
{
  return new BasicKinematicParticle ( *this );
}

bool BasicKinematicParticle::operator< ( const BasicKinematicParticle & o ) const
{
  return ( id() < o.id() );
}

BasicKinematicParticle::BasicKinematicParticle (
  boost::any particle, boost::any link,
  std::string tag ) :
    theParticle ( particle ), theId ( RaveId::uniqueId() ), theLink ( link ),
    theTag ( tag ), theIsValid ( true ),
    theState ( CmsToRaveObjects().convert ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->currentState().kinematicParameters() ) ),
    theError ( CmsToRaveObjects().convert ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->currentState().kinematicParametersError() ) ),
    theCharge ( boost::numeric_cast<Charge> ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->currentState().particleCharge() ) ),
    theChi2 ( boost::numeric_cast<double> ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->chiSquared() ) ),
    theNdof ( boost::numeric_cast<double> ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->degreesOfFreedom() ) )
{}

BasicKinematicParticle::BasicKinematicParticle (
  int id, boost::any particle, boost::any link,
  std::string tag ) :
    theParticle ( particle ), theId ( id ), theLink ( link ),
    theTag ( tag ), theIsValid ( true ),
    theState ( CmsToRaveObjects().convert ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->currentState().kinematicParameters() ) ),
    theError ( CmsToRaveObjects().convert ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->currentState().kinematicParametersError() ) ),
    theCharge ( boost::numeric_cast<Charge> ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->currentState().particleCharge() ) ),
    theChi2 ( boost::numeric_cast<double> ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->chiSquared() ) ),
    theNdof ( boost::numeric_cast<double> ( boost::any_cast< ::RefCountedKinematicParticle > ( particle )->degreesOfFreedom() ) )
{}

BasicKinematicParticle::BasicKinematicParticle () :
    theId ( -1 ), theIsValid ( false )
{}

BasicKinematicParticle::~BasicKinematicParticle()
{}

boost::any BasicKinematicParticle::getInternal() const
{
  return theParticle;
}

void BasicKinematicParticle::setInternal ( const boost::any & o ) const
{
  theParticle = o;
}

}
