#include "RaveBase/RaveInterface/rave/BasicTransientTrackKinematicParticle.h"

#include "RaveBase/Converters/interface/RaveToCmsObjects.h"
#include "RaveBase/RaveEngine/interface/TaggedTransientTrackKinematicParticle.h"
#include "RecoVertex/KinematicFitPrimitives/interface/TransientTrackKinematicParticle.h"

#include "boost/assert.hpp"
#include "boost/cast.hpp"

namespace rave
{

BasicTransientTrackKinematicParticle * BasicTransientTrackKinematicParticle::clone() const
{
  return new BasicTransientTrackKinematicParticle ( *this );
}

BasicTransientTrackKinematicParticle::BasicTransientTrackKinematicParticle (
  boost::any particle, boost::any origin, std::string tag ) :
    BasicKinematicParticle ( particle, origin, tag )
{
  BOOST_ASSERT ( typeid ( ::RefCountedKinematicParticle ) == particle.type() );
  BOOST_ASSERT (
    dynamic_cast< ::TransientTrackKinematicParticle * > (
      boost::any_cast< ::RefCountedKinematicParticle > ( particle ).get() ) );
}

BasicTransientTrackKinematicParticle::BasicTransientTrackKinematicParticle (
  int id, boost::any particle, boost::any origin, std::string tag ) :
    BasicKinematicParticle ( id, particle, origin, tag )
{
  BOOST_ASSERT ( typeid ( ::RefCountedKinematicParticle ) == particle.type() );
}

namespace
{
  boost::shared_ptr< ::KinematicStatePropagator > propagator( new ::TrackKinematicStatePropagator() );
::TransientTrackKinematicStateBuilder builder;

::RefCountedKinematicParticle createInternalParticle (
  const rave::Track & initialTrack, const double & massGuess,
  const double & massSigma, const double & chiSquared,
  const double & degreesOfFr )
{
  // Convert rave::Track to reco::TransientTrack
  boost::shared_ptr< reco::TransientTrack > recoTrack (
    new reco::TransientTrack ( RaveToCmsObjects().tTrack ( initialTrack ) ) );

  ::KinematicState initState = builder (
                                 *recoTrack,
                                 boost::numeric_cast< ::ParticleMass > ( massGuess ),
                                 boost::numeric_cast< float > ( massSigma ) );

  float chi2 = boost::numeric_cast< float > ( chiSquared );
  float ndof = boost::numeric_cast< float > ( degreesOfFr );
  ::KinematicConstraint * lastConstraint = 0;
  ReferenceCountingPointer< ::KinematicParticle > previousParticle = 0;

  ::RefCountedKinematicParticle
  taggedParticle ( new ::TaggedTransientTrackKinematicParticle (
                     initState,
                     chi2,
                     ndof,
                     lastConstraint,
                     previousParticle,
                     propagator.get(),
                     recoTrack ) );

  return taggedParticle;
}

::RefCountedKinematicParticle createInternalParticle (
  const rave::Track & initialTrack, const double & massGuess,
  const double & massSigma )
{
  return createInternalParticle ( initialTrack, massGuess, massSigma,
                                  initialTrack.chi2(), initialTrack.ndof() );
}

}

BasicTransientTrackKinematicParticle::BasicTransientTrackKinematicParticle (
  const rave::Track & initialTrack, const double & massGuess,
  const double & massSigma, const double & chiSquared,
  const double & degreesOfFr, boost::any origin, std::string tag ) :
    BasicKinematicParticle (
      createInternalParticle ( initialTrack, massGuess, massSigma, chiSquared,
                               degreesOfFr ),
      origin, tag )
{}

BasicTransientTrackKinematicParticle::BasicTransientTrackKinematicParticle (
  const rave::Track & initialTrack, const double & massGuess,
  const double & massSigma, boost::any origin, std::string tag ) :
    BasicKinematicParticle (
      createInternalParticle ( initialTrack, massGuess, massSigma ),
      origin, tag )
{}

BasicTransientTrackKinematicParticle::BasicTransientTrackKinematicParticle() :
    BasicKinematicParticle()
{}

BasicTransientTrackKinematicParticle::~BasicTransientTrackKinematicParticle()
{}

}
