#include "RaveBase/RaveInterface/rave/Covariance7D.h"

namespace rave
{

Covariance7D::Covariance7D ( double dxx, double dxy, double dxz,
                             double dyy, double dyz, double dzz,
                             double dxpx, double dxpy, double dxpz,
                             double dypx, double dypy, double dypz,
                             double dzpx, double dzpy, double dzpz,
                             double dpxpx, double dpxpy, double dpxpz,
                             double dpypy, double dpypz, double dpzpz,
                             double dxm, double dym, double dzm,
                             double dpxm, double dpym, double dpzm,
                             double dmm ) :
    Covariance6D ( dxx, dxy, dxz, dyy, dyz, dzz,
                   dxpx, dxpy, dxpz, dypx, dypy, dypz, dzpx, dzpy, dzpz,
                   dpxpx, dpxpy, dpxpz, dpypy, dpypz, dpzpz ),
    thedxm ( dxm ), thedym ( dym ), thedzm ( dzm ),
    thedpxm ( dpxm ), thedpym ( dpym ), thedpzm ( dpzm ), thedmm ( dmm )
{}

Covariance7D::Covariance7D ( Covariance6D space_error,
                             double dxm, double dym, double dzm,
                             double dpxm, double dpym, double dpzm,
                             double dmm ) :
    Covariance6D ( space_error ),
    thedxm ( dxm ), thedym ( dym ), thedzm ( dzm ),
    thedpxm ( dpxm ), thedpym ( dpym ), thedpzm ( dpzm ), thedmm ( dmm )
{}

Covariance7D::Covariance7D() :

    Covariance6D(), thedxm ( 0. ), thedym ( 0. ), thedzm ( 0. ),
    thedpxm ( 0. ), thedpym ( 0. ), thedpzm ( 0. ), thedmm ( 0. )
{}

double Covariance7D::dxm() const
{
  return thedxm;
}

double Covariance7D::dym() const

{
  return thedym;
}

double Covariance7D::dzm() const

{
  return thedzm;
}

double Covariance7D::dpxm() const

{
  return thedpxm;
}

double Covariance7D::dpym() const

{
  return thedpym;
}

double Covariance7D::dpzm() const

{
  return thedpzm;
}

double Covariance7D::dmm() const

{
  return thedmm;
}

bool Covariance7D::equal ( const Covariance7D & other ) const

{
  return Covariance6D::equal ( other ) &&
         ( dxm() == other.dxm() ) &&
         ( dym() == other.dym() ) &&
         ( dzm() == other.dzm() ) &&
         ( dpxm() == other.dpxm() ) &&
         ( dpym() == other.dpym() ) &&
         ( dpzm() == other.dpzm() ) &&
         ( dmm() == other.dmm() );
}

} // namespace rave

bool operator== ( const rave::Covariance7D & lhs, const rave::Covariance7D & rhs )
{
  return lhs.equal ( rhs );
}

std::ostream & operator << ( std::ostream & os, const rave::Covariance7D & p )
{
  os
      << "(" << p.dxx()  << "    " << p.dxy()  << "    " << p.dxz()  << "    " << p.dxpx()  << "    " << p.dxpy()  << "    " << p.dxpz()  << "    " << p.dxm()  << std::endl
      << "(" << p.dxy()  << "    " << p.dyy()  << "    " << p.dyz()  << "    " << p.dypx()  << "    " << p.dypy()  << "    " << p.dypz()  << "    " << p.dym()  << std::endl
      << "(" << p.dxz()  << "    " << p.dyz()  << "    " << p.dzz()  << "    " << p.dzpx()  << "    " << p.dzpy()  << "    " << p.dzpz()  << "    " << p.dzm()  << std::endl
      << "(" << p.dxpx() << "    " << p.dypx() << "    " << p.dzpx() << "    " << p.dpxpx() << "    " << p.dpxpy() << "    " << p.dpxpz() << "    " << p.dpxm() << std::endl
      << "(" << p.dxpy() << "    " << p.dypy() << "    " << p.dzpy() << "    " << p.dpxpy() << "    " << p.dpypy() << "    " << p.dpypz() << "    " << p.dpym() << std::endl
      << "(" << p.dxpz() << "    " << p.dypz() << "    " << p.dzpz() << "    " << p.dpxpz() << "    " << p.dpypz() << "    " << p.dpzpz() << "    " << p.dpzm() << std::endl
      << "(" << p.dxm()  << "    " << p.dym()  << "    " << p.dzm()  << "    " << p.dpxm()  << "    " << p.dpym()  << "    " << p.dpzm()  << "    " << p.dmm()  << ")" << std::endl;
  return os;
}
